#
# Copyright (c) Ensim Corporation, 2003   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#
import deployables_lib as dl
import urlparse
import shutil
import os

dl.load_msgs_module("b2-0.6.2.2", symtab = globals())

dir_moves = [('b2', '')]
file_mods = [{'path' : 'images/', 'create' : '', 'delete' : ''}]
secure_files = ['b2config.php', ('images', 0755, 0777)]
instance_patches = []

pat = "^\s*\$(?P<key>[^= ]+)\s*=\s*(?P<value>'?.*'?)\s*;(?P<comment>\s*(/\*|//).*)?\s*$"
sub = "$%(key)s = %(value)s;%(comment)s"

def install(site, site_conf, instance_conf, msgs):

    actions = ['resolve_location', 'check_disk_quota', 'check_mysql_enabled', 
	       'check_pass', 'check_install_loc_owner', '']
    args_dict = {}
    dl.do_install_actions(site, site_conf, instance_conf, msgs,
                          globals(), actions, args_dict)
    if msgs.get_severity() != msgs.ERROR:
        vars = [('siteurl', instance_conf['url_loc']),
                ('pathserver', instance_conf['url_loc']),
                ('admin_email', instance_conf['instance_admin_email']),
                ('dbname', instance_conf['dbname']),
                ('dbhost', dl.get_mysql_host(site)),
                ('dbusername', site_conf['mysql']['dbaseadmin']),
                ('dbpassword', instance_conf['dbpass']),
                ('use_fileupload', 1),
                ('fileupload_realpath', (instance_conf['install_loc'] + '/images', 
                                         dl.MODSTR_JAIL)),
                ('fileupload_url', instance_conf['url_loc'] + '/images')]
        sql_stmt1 = ["""
            UPDATE b2users SET user_pass = '%(instance_pass1)s',
            user_login = '%(instance_admin)s'
            WHERE user_login = 'admin'
            """ % instance_conf]
        actions = ['db_create', 'db_probe', 'archive_unpack', 'check_index',
                   'edit_conf_file', 'exec_php_script', 'exec_mysql',
                   'handle_security_mode', 'remove_files', 'access_url']
        args_dict = {
            'archive_unpack':       {'dir_moves': dir_moves,
                                     'file_mods' : file_mods},
            'handle_security_mode': {'files': secure_files},
            'edit_conf_file' : {'file' : 'b2config.php', 'lang' : 'php',
                                'pat' : pat, 'sub' : sub, 'vars' : vars},
            'exec_php_script' : {
                 'script' : 'b2install.php', 'args' : {},
                 'success_re' : '.*Installation successful'},
            'exec_mysql' : {'sql' : sql_stmt1},
            'remove_files': { 'files': ['b2install.php'] },
            }
        dl.do_install_actions(site, site_conf, instance_conf, msgs,
                              globals(), actions, args_dict)

def reconfigure(context, site, site_conf, old_site_conf, instance_conf,
                old_instance_conf, msgs):

    actions = ['handle_url_loc', 'resolve_location', '', 'archive_unpack',
               'handle_username_change', 'handle_db_changes']
    args_dict = {
            'archive_unpack':       { 'dir_moves': dir_moves,
                                      'file_mods' : file_mods,
                                      'fileinfo' :
                                      {'excluded_files' :
                                       ['b2install.php', 'b2config.php']}},
            }
    if context != dl.CONTEXT_SITE_EDIT and instance_conf.get('dbpass'):
        actions.insert(0, 'db_probe')
        args_dict['db_probe'] = {'use_old' : 1}
    dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                              instance_conf, old_instance_conf, msgs,
                              globals(), actions, args_dict)
    if msgs.get_severity() != msgs.ERROR:
        old_fileupload = "%s%s%s" % (dl.site_fs_jail(site, old_site_conf),
                                     instance_conf['install_loc'], '/images')
        new_fileupload = "%s%s%s" % (dl.site_fs_jail(site, site_conf),
                                     instance_conf['install_loc'], '/images')
        vars = [('siteurl', instance_conf['url_loc'], 
                        old_instance_conf['url_loc']),
                ('pathserver', instance_conf['url_loc'],
                           old_instance_conf['url_loc']),
                ('fileupload_realpath', new_fileupload, old_fileupload),
                ('fileupload_url', instance_conf['url_loc'] + '/images',
                               old_instance_conf['url_loc'] + '/images')]
        if dl.mysql_installed(site_conf, old_site_conf):
            vars.extend([('dbname', instance_conf['dbname'],
                          old_instance_conf['dbname']),
                         ('dbusername', site_conf['mysql']['dbaseadmin'],
                          old_site_conf['mysql']['dbaseadmin'])])
        if context != dl.CONTEXT_SITE_EDIT:
            # We will only change the mysql host if a user specifically
            # invokes reconfigure; we don't want this change to happen
            # during upgrade, when all the sites are edited, but the
            # mysql.sock file hasn't moved yet.
            vars.append(('dbhost', dl.get_mysql_host(site)))
            if instance_conf.get('dbpass'):
                vars.append(('dbpassword', instance_conf['dbpass']))
        actions2 = ['edit_conf_file', 'apply_patch', 'handle_security_mode']
        args_dict = {
            'handle_security_mode': { 'files': secure_files },
            'edit_conf_file' : {'file' : 'b2config.php', 'lang' : 'php',
                                'pat' : pat, 'sub' : sub, 'vars' : vars},
            'apply_patch' : {'patches' : instance_patches}
            }
        dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                                  instance_conf, old_instance_conf, msgs,
                                  globals(), actions2, args_dict)
        
        
def remove(site, site_conf, instance_conf, msgs):
    dl.db_drop(site, site_conf, instance_conf)
    dl.install_loc_cleanup(site, site_conf, instance_conf, dir_moves, 
                           file_mods = file_mods)
    
def get_instance_configuration(context, site, site_conf, instance_conf,
                               old_instance_conf, locale, msgs):
    instance = dl.dsp_create_instance(instance_conf['shortname'],
                                      instance_conf['version'])
    dl.dsp_fill_instance(context, instance, site, site_conf,
                         instance_conf, old_instance_conf,
                         ['dbname', 'username', 'url_net', 'url_path',
                          'domain_to_path', 'instance_admin_email'])
    dl.enable_patches(instance, instance_patches)
    return instance

def regression_test(action, index, test_data):
    tests = [{'disp' : 'Logging in to b2',
              'test' : 'verify_login'},
             {'disp' : 'Verifying expected page contents',
              'test' : 'check_output'}
             ]
    ret = dl.regression_check_index(action, index, test_data,
                                    'This is the first post.')
    if ret == '':
        index = index - test_data['regression_check_index']
        if action == dl.REGRESSION_GET_TEST_STRING:
            if index >= len(tests):
                return ''
            else:
                return tests[index]['disp']
        else:
            status = -1
            if tests[index]['test'] == 'verify_login':
                url = dl.add_slash(test_data['url_loc']) + 'b2login.php'
                opts = [('-F', 'redirect_to=b2edit.php'), ('-F', 'action=login'),
                        ('-F', 'log=admin'), ('-F', 'pwd=ensim'), '-L']
                status, test_data['output'] = dl.exec_curl_cmd(url, opts)
            elif tests[index]['test'] == 'check_output':
                test_data['success_re'] = 'This is the first post.'
                status = dl.check_output(test_data['output'],
                                         test_data['success_re'])
            return status
    else:
        return ret
        
        
        
