#
# Copyright (c) Ensim Corporation, 2003   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#
import deployables_lib as dl
import urlparse
import shutil
import os

dl.load_msgs_module("cslivehelp-2.7", symtab = globals())

dir_moves = [('CSLHv27', '')]
file_mods = []
secure_files = ['config.php']
instance_patches = []

pat = "^\s*\$(?P<key>\S+\w)\s*=\s*(?P<value>['\"].*?['\"])(?P<remainder>.*)"
sub = "$%(key)s = %(value)s%(remainder)s"

def install(site, site_conf, instance_conf, msgs):

    actions = ['resolve_location', 'check_disk_quota', 'check_mysql_enabled', 
	       'check_pass', 'check_install_loc_owner', '']
    args_dict = {}
    dl.do_install_actions(site, site_conf, instance_conf, msgs,
                          globals(), actions, args_dict)
    if msgs.get_severity() != msgs.ERROR:
        actions = ['db_create', 'db_probe', 'archive_unpack', 'check_index',
                   'exec_php_script', 'handle_security_mode', 'access_url']
        args_dict = {
            'archive_unpack':       {'dir_moves': dir_moves},
            'handle_security_mode': {'files': secure_files},
            'exec_php_script': {'script' : 'setup.php',
                                'args' : {'manualinstall' : '',
                                          'speaklanguage' : 'eng',
                                          'installationtype' : 'newinstall',
                                          'homepage' : instance_conf['url_loc'],
                                          'username' : instance_conf['instance_admin'],
                                          'password' : instance_conf['instance_pass1'],
                                          'password2' : instance_conf['instance_pass1'],
                                          'email' : instance_conf['instance_admin_email'],
                                          'rootpath' : (instance_conf['install_loc'], dl.MODSTR_JAIL),
                                          'dbtype' : 'mysql_options.php',
                                          'server' : dl.get_mysql_host(site),
                                          'database' : instance_conf['dbname'],
                                          'datausername' : site_conf['mysql']['dbaseadmin'],
                                          'mypassword' : instance_conf['dbpass'],
                                          'action' : 'INSTALL'},
                                'success_re' : '.*INSTALLATION IS DONE'},
            }
        dl.do_install_actions(site, site_conf, instance_conf, msgs,
                              globals(), actions, args_dict)

def reconfigure(context, site, site_conf, old_site_conf, instance_conf,
                old_instance_conf, msgs):

    actions = ['handle_url_loc', 'resolve_location', 'require_url_loc', '',
               'archive_unpack', 'handle_username_change', 'handle_db_changes']
    args_dict = {
            'archive_unpack': {'dir_moves': dir_moves,
			       'fileinfo' : {'excluded_files' :
					     ['config.php']}},
            }
    if context != dl.CONTEXT_SITE_EDIT and instance_conf.get('dbpass'):
        actions.insert(0, 'db_probe')
        args_dict['db_probe'] = {'use_old' : 1}
    dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                              instance_conf, old_instance_conf, msgs,
                              globals(), actions, args_dict)
    if msgs.get_severity() != msgs.ERROR:
        old_root = "%s%s" % (dl.site_fs_jail(site, old_site_conf),
                             instance_conf['install_loc'] + '/')
        new_root = "%s%s" % (dl.site_fs_jail(site, site_conf),
                             instance_conf['install_loc'] + '/')        
        vars = [('application_root', new_root, old_root),
                ('API_HOME_DIR', new_root, old_root)]
        if dl.mysql_installed(site_conf, old_site_conf):
            vars.extend([('datausername', site_conf['mysql']['dbaseadmin'],
                          old_site_conf['mysql']['dbaseadmin']),
                         ('database', instance_conf['dbname'],
                          old_instance_conf['dbname'])])
        if context != dl.CONTEXT_SITE_EDIT:
            # We will only change the mysql host if a user specifically
            # invokes reconfigure; we don't want this change to happen
            # during upgrade, when all the sites are edited, but the
            # mysql.sock file hasn't moved yet.
            vars.append(('server', dl.get_mysql_host(site)))
            if instance_conf.get('dbpass'):
                vars.append(('password', instance_conf['dbpass']))
        actions2 = ['edit_conf_file', 'apply_patch', 'handle_security_mode']
        args_dict = {
            'handle_security_mode': { 'files': secure_files },
            'edit_conf_file' : {'file' : 'config.php', 'lang' : 'php',
                                'pat' : pat, 'sub' : sub, 'vars' : vars},
            'apply_patch' : {'patches' : instance_patches}
            }
        dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                                  instance_conf, old_instance_conf, msgs,
                                  globals(), actions2, args_dict)
        
        
def remove(site, site_conf, instance_conf, msgs):
    dl.db_drop(site, site_conf, instance_conf)
    dl.install_loc_cleanup(site, site_conf, instance_conf, dir_moves,
                           file_mods = file_mods)
    
def get_instance_configuration(context, site, site_conf, instance_conf,
                               old_instance_conf, locale, msgs):
    instance = dl.dsp_create_instance(instance_conf['shortname'],
                                      instance_conf['version'])
    dl.dsp_fill_instance(context, instance, site, site_conf,
                         instance_conf, old_instance_conf,
                         ['dbname', 'username', 'url_net', 'url_path',
                          'domain_to_path', 'instance_admin_email'])
    dl.enable_patches(instance, instance_patches)
    return instance

def regression_test(action, index, test_data):
    test_data['allow_redirect'] = 1
    return dl.regression_check_index(action, index, test_data,
                                     'CSLH OPERATOR LOGIN')
