#
# Copyright (c) Ensim Corporation, 2003   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#
import deployables_lib as dl

dl.load_msgs_module("formmail-4.2b", symtab = globals())

dir_moves = [('', '')]
instance_patches = []

pat = '^\s*\$(?P<key>\S+)\s*=\s*(?P<value>array\s*\(.*\))\s*;\s*$'
sub = '$%(key)s = %(value)s;\r'

pat2 = '^\s*<input\s*type="hidden"\s*name="(?P<key>[^"]+)"\s*value=(?P<value>".*")>\s*$'
sub2 = '<input type="hidden" name="%(key)s" value=%(value)s>\r'

def install(site, site_conf, instance_conf, msgs):

    # Resolve location so that we can put that in place of path_conf_file
    # in demo_html.html
    dl.do_install_actions(site,site_conf,instance_conf,msgs,
                          globals(), ['resolve_location',
                                      'check_disk_quota',
                                      'check_install_loc_owner', ''],
                          {})
    if msgs.get_severity() != msgs.ERROR:
        domain = site_conf['siteinfo']['domain']
        email = instance_conf['username'] + '@' + domain
        vars = [('allowed_email_recipients_array', [domain]),
                ('referers', [domain]),
                ('banlist', [])]

        vars2 = [('recipient', email),
                 ('path_to_file',instance_conf['install_loc'])]

        actions = ['archive_unpack', 'edit_conf_file', 'edit_conf_file2']
        args_dict = {
            'archive_unpack':  { 'dir_moves': dir_moves },
            'edit_conf_file':  { 'file': 'formmail.php', 'lang': 'php',
                                 'pat': pat, 'sub': sub, 'vars': vars },
            'edit_conf_file2': { 'file': 'demo_html.html', 'lang': 'html',
                                 'pat': pat2, 'sub': sub2, 'vars': vars2 },
            }
        dl.do_install_actions(site, site_conf, instance_conf, msgs,
                              globals(), actions, args_dict)

# The following site configuration parameters can change, and affect
# the instance parameters mentioned in parentheses
# site_conf['siteinfo']['domain']   (url_loc)
# site_conf['siteinfo']['admin_user'] (username)
def reconfigure(context, site, site_conf, old_site_conf, instance_conf,
                old_instance_conf, msgs):
    old_domain = old_site_conf['siteinfo']['domain']
    new_domain =     site_conf['siteinfo']['domain']
    old_email =  old_instance_conf['username'] + '@' + old_domain
    new_email =      instance_conf['username'] + '@' + new_domain

    vars = [('allowed_email_recipients_array', [new_domain], [old_domain]),
            ('referers',                       [new_domain], [old_domain])]
    vars2 = [('recipient', new_email, old_email)]

    actions = ['handle_url_loc', 'resolve_location', '',
               'archive_unpack', 'handle_username_change',
               'edit_conf_file', 'edit_conf_file2', 'apply_patch']
    args_dict = {
        'archive_unpack':  { 'dir_moves': dir_moves,
                             'fileinfo' :
                             {'excluded_files' :
                              ['formmail.php', 'demo_html.html']}},
        'edit_conf_file':  { 'file': 'formmail.php', 'lang': 'php',
                             'pat': pat, 'sub': sub, 'vars': vars },
        'edit_conf_file2': { 'file': 'demo_html.html', 'lang': 'html',
                             'pat': pat2, 'sub': sub2, 'vars': vars2 },
        'apply_patch' : {'patches' : instance_patches}
        }
    dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                              instance_conf, old_instance_conf, msgs,
                              globals(), actions, args_dict)
        
def remove(site, site_conf, instance_conf, msgs):
    dl.install_loc_cleanup(site, site_conf, instance_conf, dir_moves)
    
def get_instance_configuration(context, site, site_conf, new_instance_conf,
                               old_instance_conf, locale, msgs):
    instance = dl.dsp_create_instance(new_instance_conf['shortname'],
                                      new_instance_conf['version'])
    dl.dsp_fill_instance(context, instance, site, site_conf,
                         new_instance_conf, old_instance_conf,
                         ['username', 'url_net', 'url_path',
                          'domain_to_path'])
    dl.enable_patches(instance, instance_patches)
    return instance

def regression_test(action, index, test_data):
    tests = [{'disp' : 'Submitting form to %s' % test_data['url_loc'] + \
              '/formmail.php',
              'test' : 'submit_form'},
             {'disp' : 'Verifying expected page contents',
              'test' : 'check_output'}
             ]

    if action == dl.REGRESSION_GET_TEST_STRING:
        if index >= len(tests):
            return ''
        else:
            return tests[index]['disp']
    else:
        status = -1
        if tests[index]['test'] == 'submit_form':
            url = dl.add_slash(test_data['url_loc']) + 'formmail.php'
            opts = [('-e', '%sdemo_html.html' % url),
                    ('-F', 'recipient=%s@%s' % \
                     (test_data['site_admin'], test_data['site']))]
            status, test_data['output'] = dl.exec_curl_cmd(url, opts)
        elif tests[index]['test'] == 'check_output':
            test_data['success_re'] = 'Thank you for your submission'
            status = dl.check_output(test_data['output'],
                                     test_data['success_re'])
        return status
