#
# Copyright (c) Ensim Corporation, 2003   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#
import deployables_lib as dl
import urlparse

dl.load_msgs_module("oscommerce-2.2ms2", symtab = globals())

dir_moves = [('oscommerce-2.2ms2/catalog', ''),
             ('oscommerce-2.2ms2', 'archive'),
             (None, 'admin/backups')]

file_mods = [{'path' : 'admin/.htpasswd', 'delete' : ''}]
source_patches = [{'file' : 'oscommerce_html_output.patch.ensim'}]
                
upgrade_patches = [{'file' : 'oscommerce_html_output.patch.ensim',
                    'args' : '-N -s -p3'}]
instance_patches = []

def install(site, site_conf, instance_conf, msgs):

    actions = ['resolve_location', 'check_disk_quota', 'check_mysql_enabled',
               'check_install_loc_owner', 'check_pass','']
    args_dict = {}
    dl.do_install_actions(site, site_conf, instance_conf, msgs,
                          globals(), actions, args_dict)
    if msgs.get_severity() != msgs.ERROR:
        split = urlparse.urlsplit(instance_conf['url_loc'])
        server_name = split[1]
        script_path = split[2]
        https_url_loc = urlparse.urlunsplit(('https', split[1], split[2],
                                             split[3], split[4]))

        sql_stmt = "UPDATE configuration SET configuration_value = '%s' " + \
                   "WHERE configuration_key = '%s'"
        sql_stmt1 = sql_stmt % (instance_conf['instance_admin_name'],
                                'STORE_OWNER')
        sql_stmt2 = sql_stmt % (instance_conf['instance_admin_email'],
                                'STORE_OWNER_EMAIL_ADDRESS')
        sql_stmt3 = sql_stmt % ("%s <%s>" % ( \
            instance_conf['instance_admin_name'],
            instance_conf['instance_admin_email']), 'EMAIL_FROM')
        
        actions = ['db_create', 'db_probe', 'archive_unpack', 'check_index',
                   'exec_php_script', 'exec_php_script2', 'exec_mysql',
                   'create_htpasswd', 'set_htpasswd',
                   'handle_security_mode', 'remove_files', 'access_url']
        args_dict = {
            'archive_unpack':        { 'dir_moves': dir_moves },
            'exec_php_script' : {
                'script': 'install/install.php',
                'args': [
                    ('step' , '3'),
                    ('install[]' , 'database'),
                    ('install[]' , 'configure'),
                    ('DB_SERVER' , dl.get_mysql_host(site)),
                    ('DB_SERVER_USERNAME' , site_conf['mysql']['dbaseadmin']),
                    ('DB_SERVER_PASSWORD' , instance_conf['dbpass']),
                    ('DB_DATABASE' , instance_conf['dbname'])],
                'success_re' : '.*was <b>successful!</b>'},
            'exec_php_script2' : {
                'script': 'install/install.php',
                'args': [
                    ('step' , '7'),
                    ('install[]' , 'database'),
                    ('install[]' , 'configure'),
                    ('DB_SERVER' , dl.get_mysql_host(site)),
                    ('DB_SERVER_USERNAME' , site_conf['mysql']['dbaseadmin']),
                    ('DB_SERVER_PASSWORD' , instance_conf['dbpass']),
                    ('DB_DATABASE' , instance_conf['dbname']),
                    ('HTTPS_WWW_ADDRESS' , https_url_loc),
                    ('HTTP_WWW_ADDRESS' , instance_conf['url_loc']),
                    ('DIR_FS_DOCUMENT_ROOT',(instance_conf['install_loc'] +'/',
                                             dl.MODSTR_JAIL)),
                    ('HTTPS_COOKIE_DOMAIN' , server_name),
                    ('HTTPS_COOKIE_PATH' , script_path),
                    ('HTTP_COOKIE_DOMAIN' , server_name),
                    ('HTTP_COOKIE_PATH' , script_path),
                    ('ENABLE_SSL' , 'true'),
                    ('STORE_SESSIONS' , 'mysql'),
                    ('USE_PCONNECT' , 'false')],
                'success_re' : '.*was successful!'},
            'exec_mysql' : { 'sql' : [sql_stmt1, sql_stmt2, sql_stmt3] },
            'create_htpasswd' : {
                 'realm' : "osCommerce Administration Page",
                 'directory' : instance_conf['install_loc'] + '/admin'},
            'set_htpasswd' : {
                 'realm' : "osCommerce Administration Page",
                 'directory' : instance_conf['install_loc'] + '/admin'},
            'handle_security_mode':
                         { 'files': [('includes/configure.php', 0400, 0644),
                                     'admin/includes/configure.php',
                                     ('admin/backups', 0700, 0777)] },
            'remove_files' : { 'files': ['install/'] },
            }
        dl.do_install_actions(site, site_conf, instance_conf, msgs,
                              globals(), actions, args_dict)

def reconfigure(context, site, site_conf, old_site_conf,
                instance_conf, old_instance_conf, msgs):
    actions = ['handle_url_loc', 'resolve_location', '',
               'archive_unpack', 'handle_username_change',
               'handle_db_changes']
    args_dict = {
        'archive_unpack': { 'dir_moves': dir_moves,
                            'fileinfo' :
                            {'excluded_files' :
                             ['includes/configure.php',
                              'admin/includes/configure.php',
                              'install', 'admin/.htaccess']}},
        }
    if context != dl.CONTEXT_SITE_EDIT and instance_conf.get('dbpass'):
        actions.insert(0, 'db_probe')
        args_dict['db_probe'] = {'use_old' : 1}
    dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                              instance_conf, old_instance_conf, msgs,
                              globals(), actions, args_dict)     
    if msgs.get_severity() != msgs.ERROR:
        old_domain = old_site_conf['siteinfo']['domain']
        new_domain = site_conf['siteinfo']['domain']
        def url_sub(key, url):
            if context == dl.CONTEXT_SITE_EDIT:
                return dl.url_net_sub(url, new_domain, old_domain)
            else:
                return dl.url_sub(url, instance_conf['url_loc'],
                              old_instance_conf['url_loc'])
        def path_sub(key, value):
            return dl.url_path_sub(value, instance_conf['url_loc'],
                                   old_instance_conf['url_loc'])

        old_loc = "%s%s" % (dl.site_fs_jail(site, old_site_conf),
                            old_instance_conf['install_loc'])
        new_loc = "%s%s" % (dl.site_fs_jail(site, site_conf),
                            instance_conf['install_loc'])

        pat = "^\s*define\('(?P<key>[^']+)'\s*,\s*(?P<value>'.*')\)\s*;(?P<comment>\s*//.*)?$"
        sub = "  define('%(key)s', %(value)s);%(comment)s"
        # some variables only appear in one of the config files, some are common
        vars = [('HTTP_SERVER',          url_sub),
                ('HTTPS_SERVER',         url_sub),
                ('HTTP_CATALOG_SERVER',  url_sub),
                ('HTTPS_CATALOG_SERVER', url_sub),
                ('HTTP_COOKIE_DOMAIN',   url_sub),
                ('HTTPS_COOKIE_DOMAIN',  url_sub),
                ('DIR_WS_ADMIN',         path_sub),
                ('DIR_WS_CATALOG',       path_sub),
                ('HTTP_COOKIE_PATH',     path_sub),
                ('HTTPS_COOKIE_PATH',    path_sub),
                ('DIR_WS_HTTP_CATALOG',  path_sub),
                ('DIR_WS_HTTPS_CATALOG', path_sub),
                ('DIR_FS_CATALOG',       new_loc + '/',       old_loc + '/'),
                ('DIR_FS_DOCUMENT_ROOT', new_loc + '/',       old_loc + '/'),
                ('DIR_FS_ADMIN',         new_loc + '/admin/', old_loc + '/admin/')]
        if dl.mysql_installed(site_conf, old_site_conf):
            vars.extend([('DB_SERVER_USERNAME',   site_conf['mysql']['dbaseadmin'],
                          old_site_conf['mysql']['dbaseadmin']),
                         ('DB_DATABASE',          instance_conf['dbname'],
                          old_instance_conf['dbname'])])
        if context != dl.CONTEXT_SITE_EDIT:
            # We will only change the mysql host if a user specifically
            # invokes reconfigure; we don't want this change to happen
            # during upgrade, when all the sites are edited, but the
            # mysql.sock file hasn't moved yet.
            vars.append(('DB_SERVER', dl.get_mysql_host(site)))
            if instance_conf.get('dbpass'):
                vars.append(('DB_SERVER_PASSWORD', instance_conf['dbpass']))

        actions = ['edit_conf_file', 'edit_conf_file2',
                   'apply_patch', 'handle_security_mode']

        args_dict = {
            'edit_conf_file': {  'file': 'includes/configure.php',
                                 'lang': 'php', 'pat': pat, 'sub': sub,
                                 'vars': vars },
            'edit_conf_file2': { 'file': 'admin/includes/configure.php',
                                 'lang': 'php', 'pat': pat, 'sub': sub,
                                 'vars': vars },
            'handle_security_mode':  { 'files':
                                       [('includes/configure.php', 0400, 0644),
                                        'admin/includes/configure.php',
                                        ('admin/backups', 0700, 0777)] },
            'apply_patch' : { 'patches' : instance_patches }
            }
        dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                                  instance_conf, old_instance_conf, msgs,
                                  globals(), actions, args_dict)     

def upgrade(site, site_conf, instance_conf, old_instance_conf, msgs):
    if old_instance_conf['shortname'] == 'oscommerce' and \
       old_instance_conf['version'] == '2.2ms2' and \
       old_instance_conf['dapiversion'] == '1.0':
        if old_instance_conf['dapirelease'] == '35':
            actions = ['apply_patch']
            args_dict = {'apply_patch' : {'patches' : upgrade_patches}}
            instance_conf['applypatches'] = '1'
            dl.do_upgrade_actions(site, site_conf, instance_conf,
                                  old_instance_conf, msgs,
                                  globals(), actions, args_dict) 

def remove(site, site_conf, instance_conf, msgs):
    dl.db_drop(site, site_conf, instance_conf)
    dl.install_loc_cleanup(site, site_conf, instance_conf, dir_moves,
                           file_mods = file_mods)

def get_instance_configuration(context, site, site_conf, instance_conf,
                               old_instance_conf, locale, msgs):
    instance = dl.dsp_create_instance(instance_conf['shortname'],
                                      instance_conf['version'])
    dl.dsp_fill_instance(context, instance, site, site_conf,
                         instance_conf, old_instance_conf,
                         ['dbname', 'username', 'url_net', 'url_path',
                          'domain_to_path', 'instance_admin_email'])
    dl.enable_patches(instance, instance_patches)
    return instance

def regression_test(action, index, test_data):
    tests = [{'disp' : 'Verifying administrator page of oscommerce',
              'test' : 'verify_admin'},
             {'disp' : 'Verifying expected page contents',
              'test' : 'check_output2'}
             ]
    ret = dl.regression_check_index(action, index, test_data,
                                    'This is a default setup of the osCommerce project')
    if ret == '':
        index = index - test_data['regression_check_index']
        if action == dl.REGRESSION_GET_TEST_STRING:
            if index >= len(tests):
                return ''
            else:
                return tests[index]['disp']
        else:
            status = -1
            if tests[index]['test'] == 'verify_admin':
                url = dl.add_slash(test_data['url_loc']) + 'admin/'
                opts = [('-u', 'admin:ensim')]
                status, test_data['output2'] = dl.exec_curl_cmd(url, opts)
            elif tests[index]['test'] == 'check_output2':
                test_data['success_re2'] = 'Choose an action' 
                status = dl.check_output(test_data['output2'],
                                         test_data['success_re2'])
            return status
    else:
        return ret
