#
# Copyright (c) Ensim Corporation, 2003   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#
import deployables_lib as dl
import urlparse
import shutil
import os

dl.load_msgs_module("phplinks-2.1.3.1", symtab = globals())

dir_moves = [('phplinks', '')]
patches = [{'file' : 'phplinks.patch.ensim'}]
secure_files = [('include/config.php', 0400, 0444)]
instance_patches = []

pat = """^\s*\$(?P<key>[^=\s]+)\s*=\s*(?P<value>("|')?.*("|')?)\s*;(?P<comment>\s*(/\*|//).*)?\s*$"""
sub = "$%(key)s = %(value)s;%(comment)s"

def install(site, site_conf, instance_conf, msgs):

    actions = ['resolve_location', 'check_disk_quota', 'check_mysql_enabled', 
	       'check_pass', 'check_install_loc_owner', '']
    args_dict = {}
    dl.do_install_actions(site, site_conf, instance_conf, msgs,
                          globals(), actions, args_dict)
    if msgs.get_severity() != msgs.ERROR:
        base_path = "%s%s" % (dl.site_fs_jail(site, site_conf),
                              instance_conf['install_loc'])
        vars = [('dbname', instance_conf['dbname']),
                ('dbhost', dl.get_mysql_host(site)),
                ('dbuser', site_conf['mysql']['dbaseadmin']),
                ('dbpasswd', instance_conf['dbpass'])]
        sql_stmt1 = ["""
            UPDATE settings SET Name = '%s', Email = '%s', 
            BaseURL = '%s', BasePath = '%s'""" % \
            (instance_conf['instance_admin_name'],
             instance_conf['instance_admin_email'],
             instance_conf['url_loc'],
             base_path)]
        actions = ['db_create', 'db_probe', 'archive_unpack', 'check_index',
                   'run_sql_file', 'edit_conf_file', 'exec_mysql', 
                   'handle_security_mode', 'access_url']
        args_dict = {
            'archive_unpack': {'dir_moves': dir_moves, 'patches' : patches},
            'handle_security_mode': {'files': secure_files},
            'run_sql_file' : {'file' : 'docs/NewPHPLinks.sql'},
            'edit_conf_file' : {'file' : 'include/config.php', 'lang' : 'php',
                                'pat' : pat, 'sub' : sub, 'vars' : vars},
            'exec_mysql' : {'sql' : sql_stmt1},
            }
        dl.do_install_actions(site, site_conf, instance_conf, msgs,
                              globals(), actions, args_dict)

def reconfigure(context, site, site_conf, old_site_conf, instance_conf,
                old_instance_conf, msgs):

    actions = ['handle_url_loc', 'resolve_location', '', 'archive_unpack',
               'handle_username_change', 'handle_db_changes']
    args_dict = {
            'archive_unpack': {'dir_moves': dir_moves,
                               'patches' : patches,      
                               'fileinfo' :
                                   {'excluded_files' :
                                    ['include/config.php']}},
            }
    if context != dl.CONTEXT_SITE_EDIT and instance_conf.get('dbpass'):
        actions.insert(0, 'db_probe')
        args_dict['db_probe'] = {'use_old' : 1}
    dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                              instance_conf, old_instance_conf, msgs,
                              globals(), actions, args_dict)
    if msgs.get_severity() != msgs.ERROR:
        base_path = "%s%s" % (dl.site_fs_jail(site, site_conf),
                              instance_conf['install_loc'])
        vars = []
        if dl.mysql_installed(site_conf, old_site_conf):
            vars.extend([('dbname', instance_conf['dbname'],
                          old_instance_conf['dbname']),
                         ('dbuser', site_conf['mysql']['dbaseadmin'],
                          old_site_conf['mysql']['dbaseadmin'])])
        if context != dl.CONTEXT_SITE_EDIT:
            # We will only change the mysql host if a user specifically
            # invokes reconfigure; we don't want this change to happen
            # during upgrade, when all the sites are edited, but the
            # mysql.sock file hasn't moved yet.
            vars.append(('dbhost', dl.get_mysql_host(site)))
            if instance_conf.get('dbpass'):
                vars.append(('dbpasswd', instance_conf['dbpass']))
        sql_stmt1 = ["""
            UPDATE settings SET BaseURL = '%s', BasePath = '%s'
            """ % (instance_conf['url_loc'], base_path)]
        actions2 = ['edit_conf_file', 'apply_patch', 'handle_security_mode']
        args_dict = {
            'handle_security_mode': { 'files': secure_files },
            'edit_conf_file' : {'file' : 'include/config.php', 'lang' : 'php',
                                'pat' : pat, 'sub' : sub, 'vars' : vars},
            'exec_mysql' : {'sql' : sql_stmt1},
            'apply_patch' : {'patches' : instance_patches}
            }
        dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                                  instance_conf, old_instance_conf, msgs,
                                  globals(), actions2, args_dict)
        
def remove(site, site_conf, instance_conf, msgs):
    dl.db_drop(site, site_conf, instance_conf)
    dl.install_loc_cleanup(site, site_conf, instance_conf, dir_moves)
    
def get_instance_configuration(context, site, site_conf, instance_conf,
                               old_instance_conf, locale, msgs):
    instance = dl.dsp_create_instance(instance_conf['shortname'],
                                      instance_conf['version'])
    dl.dsp_fill_instance(context, instance, site, site_conf,
                         instance_conf, old_instance_conf,
                         ['dbname', 'username', 'url_net', 'url_path',
                          'domain_to_path', 'instance_admin_email'])
    dl.enable_patches(instance, instance_patches)
    return instance

def regression_test(action, index, test_data):
    return dl.regression_check_index(action, index, test_data,
                                     'Add A Site')
