#
# Copyright (c) Ensim Corporation, 2003   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#
import deployables_lib as dl

dl.load_msgs_module("phpnuke-6.9", symtab = globals())

dir_moves = [('html', ''), ('', 'archive')]
patches = [{'file' : 'dbi.patch.ensim'},
           {'file' : 'my.patch.ensim'}]
upgrade_patches = [{'file' : 'my.patch.ensim', 'args' : '-N -s -p2'}]
instance_patches = []

pat = '^\s*\$(?P<key>\S+)\s*=\s*(?P<value>["\'].*[\'"])\s*;\s*$'
sub = '$%(key)s = %(value)s;'

def install(site, site_conf, instance_conf, msgs):

    # XXX need to change sitekey in config.php
    vars = [('dbhost', dl.get_mysql_host(site)),
            ('dbuname', site_conf['mysql']['dbaseadmin']),
            ('dbpass',  instance_conf['dbpass']),
            ('dbname',  instance_conf['dbname'])]

    actions = ['resolve_location', 'check_disk_quota', 'check_mysql_enabled', 
	       'check_pass', 'check_install_loc_owner', '', 'db_create', 
	       'db_probe', 'archive_unpack',
               'check_index', 'run_sql_file', 'edit_conf_file', 
	       'exec_php_script', 'handle_security_mode', 'access_url']
    args_dict = {
        'archive_unpack':       { 'dir_moves': dir_moves,
                                  'patches' : patches },
        'run_sql_file':         { 'file': 'archive/sql/nuke.sql' },
        'edit_conf_file':       { 'file': 'config.php', 'lang': 'php',
                                  'pat': pat, 'sub': sub, 'vars': vars },
        'exec_php_script':      {
            'script': 'admin.php',
            'args': { 'name': instance_conf['instance_admin'],
                      'url': 'http://%s/' % site_conf['siteinfo']['domain'],
                      'email': instance_conf['instance_admin_email'],
                      'pwd': instance_conf['instance_pass1'],
                      'user_new': '1',
                      'fop': 'create_first' },
            'success_re' : '.*Administration System Login'},
        'handle_security_mode': { 'files': ['config.php'] },
        }
    dl.do_install_actions(site, site_conf, instance_conf, msgs,
                          globals(), actions, args_dict)

def reconfigure(context, site, site_conf, old_site_conf,
                instance_conf, old_instance_conf, msgs):
    actions = ['handle_url_loc', 'resolve_location', '',
               'archive_unpack', 'handle_username_change',
               'handle_db_changes']
    args_dict = {
        'archive_unpack': { 'dir_moves': dir_moves,
                            'patches' : patches,
			    'fileinfo' :
			    {'excluded_files' : ['config.php']}},
        }
    if context != dl.CONTEXT_SITE_EDIT and instance_conf.get('dbpass'):
        actions.insert(0, 'db_probe')
        args_dict['db_probe'] = {'use_old' : 1}
    dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                              instance_conf, old_instance_conf, msgs,
                              globals(), actions, args_dict)     
    if msgs.get_severity() != msgs.ERROR:
        vars = []
        if dl.mysql_installed(site_conf, old_site_conf):
            vars.extend([('dbuname', site_conf['mysql']['dbaseadmin'],
                          old_site_conf['mysql']['dbaseadmin']),
                         ('dbname', instance_conf['dbname'],
                          old_instance_conf['dbname'])])
        if context != dl.CONTEXT_SITE_EDIT:
            # We will only change the mysql host if a user specifically
            # invokes reconfigure; we don't want this change to happen
            # during upgrade, when all the sites are edited, but the
            # mysql.sock file hasn't moved yet.
            vars.append(('dbhost', dl.get_mysql_host(site)))
            if instance_conf.get('dbpass'):
                vars.append(('dbpass', instance_conf['dbpass']))

        actions = ['edit_conf_file', 'apply_patch',
                   'handle_security_mode']
        args_dict = {
            'edit_conf_file': { 'file': 'config.php', 'lang': 'php',
                                'pat': pat, 'sub': sub, 'vars': vars },
            'handle_security_mode': { 'files': ['config.php'] },
            'apply_patch' : {'patches' : instance_patches}
            }
        dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                                  instance_conf, old_instance_conf, msgs,
                                  globals(), actions, args_dict)     

def upgrade(site, site_conf, instance_conf, old_instance_conf, msgs):
    if old_instance_conf['shortname'] == 'phpnuke' and \
       old_instance_conf['version'] == '6.9' and \
       old_instance_conf['dapiversion'] == '1.0' and \
       old_instance_conf['dapirelease'] in ('28', '30'):
        actions = ['apply_patch']
        args_dict = {'apply_patch' : {'patches' : upgrade_patches}}
        instance_conf['applypatches'] = '1'
        dl.do_upgrade_actions(site, site_conf, instance_conf,
                              old_instance_conf, msgs,
                              globals(), actions, args_dict)

def remove(site, site_conf, instance_conf, msgs):
    dl.db_drop(site, site_conf, instance_conf)
    dl.install_loc_cleanup(site, site_conf, instance_conf, dir_moves, patches)

def get_instance_configuration(context, site, site_conf, instance_conf,
                               old_instance_conf, locale, msgs):
    instance = dl.dsp_create_instance(instance_conf['shortname'],
                                      instance_conf['version'])

    dl.dsp_fill_instance(context, instance, site, site_conf,
                         instance_conf, old_instance_conf,
                         ['dbname', 'username', 'url_net', 'url_path',
                          'domain_to_path', 'instance_admin_email'])
    dl.enable_patches(instance, instance_patches)
    return instance

def regression_test(action, index, test_data):
    return dl.regression_check_index(action, index, test_data,
                                     'Welcome to PHP-Nuke!')
