#
# Copyright (c) Ensim Corporation, 2003   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#
import deployables_lib as dl
import urlparse

dl.load_msgs_module("phpopenchat-3.0.0b3", symtab = globals())

dir_moves = [('phpopenchat-3.0.0b3', '')]
file_mods = [{'path' : 'tmp/', 'perms' : 01777, 'create' : None, 'delete' : ''}]
instance_patches = []

pat = "^\s*define\('(?P<key>[^']+)'\s*,\s*(?P<value>'.*')\)\s*;(?P<comment>\s*/\*.*)?$"
sub = "define('%(key)s',%(value)s);%(comment)s"

def install(site, site_conf, instance_conf, msgs):

    actions = ['resolve_location', 'check_disk_quota', 'check_mysql_enabled', 
	       'check_pass', 'check_install_loc_owner', '']
    args_dict = {}
    dl.do_install_actions(site, site_conf, instance_conf, msgs,
                          globals(), actions, args_dict)
    
    if msgs.get_severity() != msgs.ERROR:
        sql_stmt1 = ["""
            UPDATE poc_user_account SET password = '%(instance_pass1)s'
            WHERE user = 'operator'
            """ % instance_conf,
                     """
            UPDATE poc_user_data SET email = '%(instance_admin_email)s',
            name = '%(instance_admin_name)s' WHERE user = 'operator'
            """ % instance_conf,
                     """
            UPDATE poc_user_groups SET member = 'a:1:{i:0;s:%d:"%s";}' 
            WHERE name = 'operator' 
            """ % (len("operator"), "operator"),
                     """
            DELETE FROM poc_user_account WHERE USER = 'mirko' OR
            user = 'frerk' OR user = 'michael'
            """,
                     """
            DELETE FROM poc_user_data WHERE USER = 'mirko' OR
            user = 'frerk' OR user = 'michael'
            """
            ]
        vars = [('DATABASE_HOST',       dl.get_mysql_host(site)),
                ('DATABASE_USER',       site_conf['mysql']['dbaseadmin']),
                ('DATABASE_PASSWORD',   instance_conf['dbpass']),
                ('DATABASE_TABLESPACE', instance_conf['dbname']),
		('ADMIN_MAIL_ADDRESS',  instance_conf['instance_admin_email']),
                ('CONFIRMATION_HOST',
                 urlparse.urlsplit(instance_conf['url_loc'])[1]),
                ('TMPDIR', 
                 (instance_conf['install_loc'] + '/tmp', dl.MODSTR_JAIL),
                 '/tmp')]

        actions = ['db_create', 'db_probe', 'archive_unpack', 'check_index', 
		   'run_sql_file', 'edit_conf_file', 'handle_security_mode', 
		   'exec_mysql', 'access_url']
        args_dict = {
	  'archive_unpack':       { 'dir_moves': dir_moves, 
				    'file_mods': file_mods },
	  'run_sql_file':         { 'file': 'db.schema' },
	  'edit_conf_file': { 'file': 'config.inc.php',
			      'lang': 'php', 'pat': pat,
			      'sub': sub, 'vars': vars },
	  'handle_security_mode': { 'files': ['config.inc.php'] },
	  'exec_mysql' : {'sql' : sql_stmt1},
	  'access_url' : {'user' : 'operator'}
	}
        dl.do_install_actions(site, site_conf, instance_conf, msgs,
                              globals(), actions, args_dict)
     

def reconfigure(context, site, site_conf, old_site_conf,
                instance_conf, old_instance_conf, msgs):
    actions = ['handle_url_loc', 'resolve_location', '',
               'archive_unpack', 'handle_username_change',
               'handle_db_changes']
    args_dict = {
        'archive_unpack': { 'dir_moves': dir_moves,
			    'fileinfo' :
			    {'excluded_files' : ['config.inc.php']} },
        }
    if context != dl.CONTEXT_SITE_EDIT and instance_conf.get('dbpass'):
        actions.insert(0, 'db_probe')
        args_dict['db_probe'] = {'use_old' : 1}
    dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                              instance_conf, old_instance_conf, msgs,
                              globals(), actions, args_dict)
    if msgs.get_severity() != msgs.ERROR:
        old_tmp = "%s%s%s" % (dl.site_fs_jail(site, old_site_conf),
                              old_instance_conf['install_loc'], '/tmp')
        new_tmp = "%s%s%s" % (dl.site_fs_jail(site, site_conf),
                              instance_conf['install_loc'], '/tmp')
        vars = [('CONFIRMATION_HOST',
                 urlparse.urlsplit(instance_conf['url_loc'])[1],
                 urlparse.urlsplit(old_instance_conf['url_loc'])[1]),
                ('TMPDIR', new_tmp, old_tmp)]
        if dl.mysql_installed(site_conf, old_site_conf):
            vars.extend([('DATABASE_USER', site_conf['mysql']['dbaseadmin'],
                          old_site_conf['mysql']['dbaseadmin']),
                         ('DATABASE_TABLESPACE', instance_conf['dbname'],
                          old_instance_conf['dbname'])])
        if context != dl.CONTEXT_SITE_EDIT:
            # We will only change the mysql host if a user specifically
            # invokes reconfigure; we don't want this change to happen
            # during upgrade, when all the sites are edited, but the
            # mysql.sock file hasn't moved yet.
            vars.append(('DATABASE_HOST', dl.get_mysql_host(site)))
            if instance_conf.get('dbpass'):
                vars.append(('DATABASE_PASSWORD', instance_conf['dbpass']))
        actions = ['edit_conf_file', 'apply_patch',
                   'handle_security_mode']
        args_dict = {
            'edit_conf_file': { 'file': 'config.inc.php', 'lang': 'php',
                                'pat': pat, 'sub': sub, 'vars': vars },
            'handle_security_mode': { 'files': ['config.inc.php'] },
            'apply_patch' : {'patches' : instance_patches}
            }
        dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                                  instance_conf, old_instance_conf, msgs,
                                  globals(), actions, args_dict)     

def remove(site, site_conf, instance_conf, msgs):
    dl.db_drop(site, site_conf, instance_conf)
    dl.install_loc_cleanup(site, site_conf, instance_conf, dir_moves,
                           file_mods = file_mods)

def get_instance_configuration(context, site, site_conf, instance_conf,
                               old_instance_conf, locale, msgs):
    instance = dl.dsp_create_instance(instance_conf['shortname'],
                                      instance_conf['version'])

    dl.dsp_fill_instance(context, instance, site, site_conf,
                         instance_conf, old_instance_conf,
                         ['dbname', 'username', 'url_net', 'url_path',
                          'domain_to_path', 'instance_admin_email'])
    dl.enable_patches(instance, instance_patches)
    return instance

def regression_test(action, index, test_data):
    return dl.regression_check_index(action, index, test_data,
                                     'Welcome to your PHPOpenChat-Installation!')
