#
# Copyright (c) Ensim Corporation, 2003   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#
import deployables_lib as dl
import base64

dl.load_msgs_module("postnuke-0.7.2.6", symtab = globals())

dir_moves = [('PostNuke-0.726/html', ''),
             ('PostNuke-0.726', 'archive')]
instance_patches = []

def install(site, site_conf, instance_conf, msgs):

    actions = ['resolve_location', 'check_disk_quota', 'check_mysql_enabled',
               'check_pass', 'check_install_loc_owner', '']
    args_dict = {}
    dl.do_install_actions(site, site_conf, instance_conf, msgs,
                          globals(), actions, args_dict)
    if msgs.get_severity() != msgs.ERROR:
        actions = ['db_create', 'db_probe', 'archive_unpack', 'check_index',
                   'exec_php_script', 'exec_php_script2',
                   'handle_security_mode', 'remove_files', 'access_url']
        args_dict = {
            'archive_unpack':       { 'dir_moves': dir_moves },
            'exec_php_script' : {
                'script' : 'install.php',
                'args' : {'op' : 'Start',
                          'currentlang' : 'eng',
                          'dbhost' : dl.get_mysql_host(site),
                          'dbuname' : site_conf['mysql']['dbaseadmin'],
                          'dbpass' : instance_conf['dbpass'],
                          'dbname' : instance_conf['dbname'],
                          'prefix' : '',
                          'dbtype' : 'mysql',
                          'dbtabletype': 'myisam',
                          'intranet' : '0'},
                'success_re' : '.*name="op" value="Continue"'},
            'exec_php_script2' : {
                'script' : 'install.php',
                'args' : {'op' : 'Set Login',
                          'currentlang' : 'eng',
                          'dbhost' : dl.get_mysql_host(site),
                          'dbuname' : site_conf['mysql']['dbaseadmin'],
                          'dbpass' : instance_conf['dbpass'],
                          'dbname' : instance_conf['dbname'],
                          'prefix' : '',
                          'dbtype' : 'mysql',
                          'intranet' : '0',
                          'aid' : instance_conf['instance_admin'],
                          'name' : instance_conf['instance_admin_name'],
                          'pwd' : instance_conf['instance_pass1'],
                          'repeatpwd' : instance_conf['instance_pass1'],
                          'email' : instance_conf['instance_admin_email'],
                          'url' : instance_conf['url_loc']},
                'success_re' : '.*name="op" value="Finish"'},
            'handle_security_mode': { 'files': ['config.php'] },
            'remove_files': { 'files': ['install.php', 'install/'] },
            }
        dl.do_install_actions(site, site_conf, instance_conf, msgs,
                              globals(), actions, args_dict)

def reconfigure(context, site, site_conf, old_site_conf,
                instance_conf, old_instance_conf, msgs):
    actions = ['handle_url_loc', 'resolve_location', '',
               'archive_unpack', 'handle_username_change',
               'handle_db_changes']
    args_dict = {
        'archive_unpack': { 'dir_moves': dir_moves,
                            'fileinfo' :
                            {'excluded_files' :
                             ['config.php', 'install.php', 'install']}},
        }
    if context != dl.CONTEXT_SITE_EDIT and instance_conf.get('dbpass'):
        actions.insert(0, 'db_probe')
        args_dict['db_probe'] = {'use_old' : 1}
    dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                              instance_conf, old_instance_conf, msgs,
                              globals(), actions, args_dict)     
    if msgs.get_severity() != msgs.ERROR:
        pat = "^\s*\$pnconfig\['(?P<key>\S+)'\]\s*=\s*(?P<value>'.*')\s*;\s*$"
        sub = "$pnconfig['%(key)s'] = %(value)s;"
        vars = []
        if dl.mysql_installed(site_conf, old_site_conf):
            vars.extend(\
                [('dbname', instance_conf['dbname'],
                  old_instance_conf['dbname']),
                 ('dbuname',
                  base64.encodestring(site_conf['mysql']['dbaseadmin']).strip(),
                  base64.encodestring(old_site_conf['mysql']['dbaseadmin']).strip())])
        if context != dl.CONTEXT_SITE_EDIT:
            # We will only change the mysql host if a user specifically
            # invokes reconfigure; we don't want this change to happen
            # during upgrade, when all the sites are edited, but the
            # mysql.sock file hasn't moved yet.
            vars.append(('dbhost', dl.get_mysql_host(site)))
            if instance_conf.get('dbpass'):
                vars.append(('dbpass',
                             base64.encodestring(instance_conf['dbpass']).strip()))

        actions = ['edit_conf_file', 'apply_patch',
                   'handle_security_mode']
        args_dict = {
            'edit_conf_file': { 'file': 'config.php', 'lang': 'php',
                                'pat': pat, 'sub': sub, 'vars': vars },
            'handle_security_mode': { 'files': ['config.php'] },
            'apply_patch' : {'patches' : instance_patches}
            }
        dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                                  instance_conf, old_instance_conf, msgs,
                                  globals(), actions, args_dict)     

def remove(site, site_conf, instance_conf, msgs):
    dl.db_drop(site, site_conf, instance_conf)
    dl.install_loc_cleanup(site, site_conf, instance_conf, dir_moves)

def get_instance_configuration(context, site, site_conf, instance_conf,
                               old_instance_conf, locale, msgs):
    instance = dl.dsp_create_instance(instance_conf['shortname'],
                                      instance_conf['version'])
    dl.dsp_fill_instance(context, instance, site, site_conf,
                         instance_conf, old_instance_conf,
                         ['dbname', 'username', 'url_net', 'url_path',
                          'domain_to_path', 'instance_admin_email'])
    dl.enable_patches(instance, instance_patches)
    return instance

def regression_test(action, index, test_data):
    return dl.regression_check_index(action, index, test_data,
                                     'Welcome to PostNuke')
